@@ -54,11 +54,34 @@
 extern gint camel_application_is_exiting;
 
 struct _CamelMapiFolderPrivate {
-
 	GStaticMutex search_lock;	/* for locking the search object */
 
+	gchar *foreign_username;
 };
 
+static gboolean
+cmf_open_folder (CamelMapiFolder *mapi_folder,
+		 EMapiConnection *conn,
+		 mapi_object_t *obj_folder,
+		 GCancellable *cancellable,
+		 GError **perror)
+{
+	gboolean res;
+
+	g_return_val_if_fail (mapi_folder != NULL, FALSE);
+	g_return_val_if_fail (conn != NULL, FALSE);
+	g_return_val_if_fail (obj_folder != NULL, FALSE);
+
+	if ((mapi_folder->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_FOREIGN) != 0)
+		res = e_mapi_connection_open_foreign_folder (conn, mapi_folder->priv->foreign_username, mapi_folder->folder_id, obj_folder, cancellable, perror);
+	else if ((mapi_folder->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_PUBLIC) != 0)
+		res = e_mapi_connection_open_public_folder (conn, mapi_folder->folder_id, obj_folder, cancellable, perror);
+	else
+		res = e_mapi_connection_open_personal_folder (conn, mapi_folder->folder_id, obj_folder, cancellable, perror);
+
+	return res;
+}
+
 /*for syncing flags back to server*/
 typedef struct {
 	guint32 changed;
@@ -236,6 +259,7 @@ struct GatherChangedObjectsData
 	GSList *to_update; /* mapi_id_t * */
 	GHashTable *removed_uids;
 	time_t latest_last_modify;
+	gboolean is_public_folder;
 };
 
 static gboolean
@@ -275,11 +299,25 @@ gather_changed_objects_to_slist (EMapiConnection *conn,
 			} else {
 				guint32 mask = CAMEL_MESSAGE_SEEN | CAMEL_MESSAGE_ATTACHMENTS, flags = 0;
 
+				/* do not change unread state for known messages in public folders */
+				if (gco->is_public_folder)
+					mask &= ~CAMEL_MESSAGE_SEEN;
+
 				if ((object_data->msg_flags & MSGFLAG_READ) != 0)
 					flags |= CAMEL_MESSAGE_SEEN;
 				if ((object_data->msg_flags & MSGFLAG_HASATTACH) != 0)
 					flags |= CAMEL_MESSAGE_ATTACHMENTS;
 
+				if ((minfo->info.flags & CAMEL_MAPI_MESSAGE_WITH_READ_RECEIPT) != 0) {
+					if ((object_data->msg_flags & MSGFLAG_RN_PENDING) == 0 &&
+					    !camel_message_info_user_flag (info, "receipt-handled")) {
+						camel_message_info_set_user_flag (info, "receipt-handled", TRUE);
+						minfo->info.dirty = TRUE;
+
+						camel_folder_summary_touch (gco->summary);
+					}
+				}
+
 				if ((minfo->info.flags & mask) != (flags & mask)) {
 					camel_message_info_set_flags (info, mask, flags);
 					minfo->server_flags = camel_message_info_flags (info);
@@ -314,10 +352,81 @@ gather_changed_objects_to_slist (EMapiConnection *conn,
 	return TRUE;
 }
 
+static void
+update_message_info (CamelMessageInfo *info,
+		     /* const */ EMapiObject *object,
+		     gboolean is_new,
+		     gboolean is_public_folder,
+		     gboolean user_has_read)
+{
+	CamelMapiMessageInfo *minfo = (CamelMapiMessageInfo *) info;
+	guint32 flags = 0, mask = CAMEL_MESSAGE_SEEN | CAMEL_MESSAGE_ATTACHMENTS | CAMEL_MESSAGE_ANSWERED | CAMEL_MESSAGE_FORWARDED | CAMEL_MAPI_MESSAGE_WITH_READ_RECEIPT;
+	const uint32_t *pmsg_flags, *picon_index;
+	const struct FILETIME *last_modified;
+	const uint8_t *pread_receipt;
+	const gchar *msg_class;
+	uint32_t msg_flags;
+
+	g_return_if_fail (info != NULL);
+	g_return_if_fail (object != NULL);
+
+	pmsg_flags = e_mapi_util_find_array_propval (&object->properties, PidTagMessageFlags);
+	last_modified = e_mapi_util_find_array_propval (&object->properties, PidTagLastModificationTime);
+	picon_index = e_mapi_util_find_array_propval (&object->properties, PidTagIconIndex);
+	pread_receipt = e_mapi_util_find_array_propval (&object->properties, PidTagReadReceiptRequested);
+	msg_class = e_mapi_util_find_array_propval (&object->properties, PidTagMessageClass);
+
+	if (msg_class && g_str_has_prefix (msg_class, "REPORT.IPM.Note.IPNRN"))
+		pread_receipt = NULL;
+
+	msg_flags = pmsg_flags ? *pmsg_flags : 0;
+
+	if (!is_new && is_public_folder) {
+		/* do not change unread state for known messages in public folders */
+		if ((user_has_read ? 1 : 0) != ((msg_flags & MSGFLAG_READ) ? 1 : 0))
+			msg_flags = (msg_flags & (~MSGFLAG_READ)) | (user_has_read ? MSGFLAG_READ : 0);
+	}
+
+	if (last_modified) {
+		minfo->last_modified = e_mapi_util_filetime_to_time_t (last_modified);
+	} else {
+		minfo->last_modified = 0;
+	}
+
+	if ((msg_flags & MSGFLAG_READ) != 0)
+		flags |= CAMEL_MESSAGE_SEEN;
+	if ((msg_flags & MSGFLAG_HASATTACH) != 0)
+		flags |= CAMEL_MESSAGE_ATTACHMENTS;
+	if (picon_index) {
+		if (*picon_index == 0x105)
+			flags |= CAMEL_MESSAGE_ANSWERED;
+		if (*picon_index == 0x106)
+			flags |= CAMEL_MESSAGE_FORWARDED;
+	}
+
+	if (pread_receipt && *pread_receipt)
+		flags |= CAMEL_MAPI_MESSAGE_WITH_READ_RECEIPT;
+
+	if (pread_receipt && *pread_receipt && (msg_flags & MSGFLAG_RN_PENDING) == 0)
+		camel_message_info_set_user_flag (info, "receipt-handled", TRUE);
+
+	if ((camel_message_info_flags (info) & mask) != flags) {
+		if (is_new)
+			minfo->info.flags = flags;
+		else
+			camel_message_info_set_flags (info, mask, flags);
+		minfo->server_flags = camel_message_info_flags (info);
+	}
+
+	minfo->info.dirty = TRUE;
+	camel_folder_summary_touch (minfo->info.summary);
+}
+
 struct GatherObjectSummaryData
 {
 	CamelFolder *folder;
 	CamelFolderChangeInfo *changes;
+	gboolean is_public_folder;
 };
 
 static void
@@ -355,24 +464,20 @@ gather_object_offline_cb (EMapiConnection *conn,
 	if (msg) {
 		gchar *uid_str;
 		const mapi_id_t *pmid;
-		const uint32_t *pmsg_flags;
-		const struct FILETIME *last_modified;
-		uint32_t msg_flags;
 		CamelMessageInfo *info;
 		gboolean is_new;
+		gboolean user_has_read = FALSE;
 
-		pmid = e_mapi_util_find_array_propval (&object->properties, PR_MID);
-		pmsg_flags = e_mapi_util_find_array_propval (&object->properties, PR_MESSAGE_FLAGS);
-		last_modified = e_mapi_util_find_array_propval (&object->properties, PR_LAST_MODIFICATION_TIME);
+		pmid = e_mapi_util_find_array_propval (&object->properties, PidTagMid);
 
 		if (!pmid) {
-			g_debug ("%s: Received message [%d/%d] without PR_MID", G_STRFUNC, obj_index, obj_total);
+			g_debug ("%s: Received message [%d/%d] without PidTagMid", G_STRFUNC, obj_index, obj_total);
 			e_mapi_debug_dump_object (object, TRUE, 3);
 			return TRUE;
 		}
 
-		if (!last_modified) {
-			g_debug ("%s: Received message [%d/%d] without PR_LAST_MODIFICATION_TIME", G_STRFUNC, obj_index, obj_total);
+		if (!e_mapi_util_find_array_propval (&object->properties, PidTagLastModificationTime)) {
+			g_debug ("%s: Received message [%d/%d] without PidTagLastModificationTime", G_STRFUNC, obj_index, obj_total);
 			e_mapi_debug_dump_object (object, TRUE, 3);
 		}
 
@@ -380,47 +485,34 @@ gather_object_offline_cb (EMapiConnection *conn,
 		if (!uid_str)
 			return FALSE;
 
-		msg_flags = pmsg_flags ? *pmsg_flags : 0;
-
 		is_new = !camel_folder_summary_check_uid (gos->folder->summary, uid_str);
-		if (!is_new)
+		if (!is_new) {
+			/* keep local read/unread flag on messages from public folders */
+			if (gos->is_public_folder) {
+				info = camel_folder_summary_get (gos->folder->summary, uid_str);
+				if (info) {
+					user_has_read = (camel_message_info_flags (info) & CAMEL_MESSAGE_SEEN) != 0;
+					camel_message_info_free (info);
+				}
+			}
+
 			camel_folder_summary_remove_uid (gos->folder->summary, uid_str);
+		}
 
 		info = camel_folder_summary_info_new_from_message (gos->folder->summary, msg, NULL);
 		if (info) {
 			CamelMapiMessageInfo *minfo = (CamelMapiMessageInfo *) info;
-			guint32 flags = 0, mask = CAMEL_MESSAGE_SEEN | CAMEL_MESSAGE_ATTACHMENTS;
 
 			minfo->info.uid = camel_pstring_strdup (uid_str);
 
-			if (last_modified) {
-				minfo->last_modified = e_mapi_util_filetime_to_time_t (last_modified);
-			} else {
-				minfo->last_modified = 0;
-			}
+			update_message_info (info, object, is_new, gos->is_public_folder, user_has_read);
 
-			if ((msg_flags & MSGFLAG_READ) != 0)
-				flags |= CAMEL_MESSAGE_SEEN;
-			if ((msg_flags & MSGFLAG_HASATTACH) != 0)
-				flags |= CAMEL_MESSAGE_ATTACHMENTS;
-
-			if ((camel_message_info_flags (info) & mask) != flags) {
-				if (is_new)
-					minfo->info.flags = flags;
-				else
-					camel_message_info_set_flags (info, mask, flags);
-				minfo->server_flags = camel_message_info_flags (info);
-			}
-
-			minfo->info.dirty = TRUE;
-			camel_folder_summary_touch (gos->folder->summary);
+			camel_folder_summary_add (gos->folder->summary, info);
+			camel_message_info_ref (info);
 
 			if (is_new) {
-				camel_folder_summary_add (gos->folder->summary, info);
 				camel_folder_change_info_add_uid (gos->changes, camel_message_info_uid (info));
 				camel_folder_change_info_recent_uid (gos->changes, camel_message_info_uid (info));
-
-				camel_message_info_ref (info);
 			} else {
 				camel_folder_change_info_change_uid (gos->changes, camel_message_info_uid (info));
 			}
@@ -457,10 +549,7 @@ gather_object_summary_cb (EMapiConnection *conn,
 	struct GatherObjectSummaryData *gos = user_data;
 	gchar *uid_str;
 	const mapi_id_t *pmid;
-	const uint32_t *pmsg_flags;
-	const struct FILETIME *last_modified;
 	const gchar *transport_headers;
-	uint32_t msg_flags;
 	CamelMessageInfo *info;
 	gboolean is_new = FALSE;
 
@@ -468,19 +557,17 @@ gather_object_summary_cb (EMapiConnection *conn,
 	g_return_val_if_fail (gos->folder != NULL, FALSE);
 	g_return_val_if_fail (object != NULL, FALSE);
 
-	pmid = e_mapi_util_find_array_propval (&object->properties, PR_MID);
-	pmsg_flags = e_mapi_util_find_array_propval (&object->properties, PR_MESSAGE_FLAGS);
-	last_modified = e_mapi_util_find_array_propval (&object->properties, PR_LAST_MODIFICATION_TIME);
-	transport_headers = e_mapi_util_find_array_propval (&object->properties, PR_TRANSPORT_MESSAGE_HEADERS_UNICODE);
+	pmid = e_mapi_util_find_array_propval (&object->properties, PidTagMid);
+	transport_headers = e_mapi_util_find_array_propval (&object->properties, PidTagTransportMessageHeaders);
 
 	if (!pmid) {
-		g_debug ("%s: Received message [%d/%d] without PR_MID", G_STRFUNC, obj_index, obj_total);
+		g_debug ("%s: Received message [%d/%d] without PidTagMid", G_STRFUNC, obj_index, obj_total);
 		e_mapi_debug_dump_object (object, TRUE, 3);
 		return TRUE;
 	}
 
-	if (!last_modified) {
-		g_debug ("%s: Received message [%d/%d] without PR_LAST_MODIFICATION_TIME", G_STRFUNC, obj_index, obj_total);
+	if (!e_mapi_util_find_array_propval (&object->properties, PidTagLastModificationTime)) {
+		g_debug ("%s: Received message [%d/%d] without PidTagLastModificationTime", G_STRFUNC, obj_index, obj_total);
 		e_mapi_debug_dump_object (object, TRUE, 3);
 	}
 
@@ -488,8 +575,6 @@ gather_object_summary_cb (EMapiConnection *conn,
 	if (!uid_str)
 		return FALSE;
 
-	msg_flags = pmsg_flags ? *pmsg_flags : 0;
-
 	info = camel_folder_summary_get (gos->folder->summary, uid_str);
 	if (!info) {
 		CamelMapiMessageInfo *minfo;
@@ -511,8 +596,12 @@ gather_object_summary_cb (EMapiConnection *conn,
 				info = camel_folder_summary_info_new_from_header (gos->folder->summary, part->headers);
 				if (info) {
 					CamelMapiMessageInfo *minfo = (CamelMapiMessageInfo *) info;
+					const uint32_t *msg_size;
 
 					minfo->info.uid = camel_pstring_strdup (uid_str);
+
+					msg_size = e_mapi_util_find_array_propval (&object->properties, PidTagMessageSize);
+					minfo->info.size = msg_size ? *msg_size : 0;
 				}
 			}
 
@@ -528,15 +617,15 @@ gather_object_summary_cb (EMapiConnection *conn,
 			gchar *formatted_addr, *from_name, *from_email;
 			CamelAddress *to_addr, *cc_addr, *bcc_addr;
 
-			subject = e_mapi_util_find_array_propval (&object->properties, PR_SUBJECT_UNICODE);
+			subject = e_mapi_util_find_array_propval (&object->properties, PidTagSubject);
 			delivery_time = e_mapi_util_find_array_propval (&object->properties, PidTagMessageDeliveryTime);
 			submit_time = e_mapi_util_find_array_propval (&object->properties, PidTagClientSubmitTime);
-			msg_size = e_mapi_util_find_array_propval (&object->properties, PR_MESSAGE_SIZE);
-			message_id = e_mapi_util_find_array_propval (&object->properties, PR_INTERNET_MESSAGE_ID);
-			references = e_mapi_util_find_array_propval (&object->properties, PR_INTERNET_REFERENCES);
-			in_reply_to = e_mapi_util_find_array_propval (&object->properties, PR_IN_REPLY_TO_ID);
-			display_to = e_mapi_util_find_array_propval (&object->properties, PR_DISPLAY_TO_UNICODE);
-			display_cc = e_mapi_util_find_array_propval (&object->properties, PR_DISPLAY_CC_UNICODE);
+			msg_size = e_mapi_util_find_array_propval (&object->properties, PidTagMessageSize);
+			message_id = e_mapi_util_find_array_propval (&object->properties, PidTagInternetMessageId);
+			references = e_mapi_util_find_array_propval (&object->properties, PidTagInternetReferences);
+			in_reply_to = e_mapi_util_find_array_propval (&object->properties, PidTagInReplyToId);
+			display_to = e_mapi_util_find_array_propval (&object->properties, PidTagDisplayTo);
+			display_cc = e_mapi_util_find_array_propval (&object->properties, PidTagDisplayCc);
 
 			info = camel_message_info_new (gos->folder->summary);
 			minfo = (CamelMapiMessageInfo *) info;
@@ -588,9 +677,9 @@ gather_object_summary_cb (EMapiConnection *conn,
 			from_email = NULL;
 
 			e_mapi_mail_utils_decode_email_address1 (conn, &object->properties,
-				PR_SENT_REPRESENTING_NAME_UNICODE,
-				PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE,
-				PR_SENT_REPRESENTING_ADDRTYPE,
+				PidTagSentRepresentingName,
+				PidTagSentRepresentingEmailAddress,
+				PidTagSentRepresentingAddressType,
 				&from_name, &from_email);
 
 			if (from_email && *from_email) {
@@ -615,30 +704,9 @@ gather_object_summary_cb (EMapiConnection *conn,
 	}
 
 	if (info) {
-		CamelMapiMessageInfo *minfo = (CamelMapiMessageInfo *) info;
-		guint32 flags = 0, mask = CAMEL_MESSAGE_SEEN | CAMEL_MESSAGE_ATTACHMENTS;
+		gboolean user_has_read = (camel_message_info_flags (info) & CAMEL_MESSAGE_SEEN) != 0;
 
-		if (last_modified) {
-			minfo->last_modified = e_mapi_util_filetime_to_time_t (last_modified);
-		} else {
-			minfo->last_modified = 0;
-		}
-
-		if ((msg_flags & MSGFLAG_READ) != 0)
-			flags |= CAMEL_MESSAGE_SEEN;
-		if ((msg_flags & MSGFLAG_HASATTACH) != 0)
-			flags |= CAMEL_MESSAGE_ATTACHMENTS;
-
-		if ((camel_message_info_flags (info) & mask) != flags) {
-			if (is_new)
-				minfo->info.flags = flags;
-			else
-				camel_message_info_set_flags (info, mask, flags);
-			minfo->server_flags = camel_message_info_flags (info);
-		}
-
-		minfo->info.dirty = TRUE;
-		camel_folder_summary_touch (gos->folder->summary);
+		update_message_info (info, object, is_new, gos->is_public_folder, user_has_read);
 
 		if (is_new) {
 			camel_folder_summary_add (gos->folder->summary, info);
@@ -691,11 +759,6 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 
 	camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
-	if ((CAMEL_MAPI_FOLDER (folder)->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_PUBLIC) != 0)
-		status = e_mapi_connection_open_public_folder (conn, mapi_folder->folder_id, &obj_folder, cancellable, mapi_error);
-	else
-		status = e_mapi_connection_open_personal_folder (conn, mapi_folder->folder_id, &obj_folder, cancellable, mapi_error);
-
 	si = camel_mapi_store_summary_get_folder_id (mapi_store->summary, mapi_folder->folder_id);
 	msi = (CamelMapiStoreInfo *) si;
 
@@ -709,6 +772,8 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 		return FALSE;
 	}
 
+	status = cmf_open_folder (mapi_folder, conn, &obj_folder, cancellable, mapi_error);
+
 	if (status) {
 		status = e_mapi_connection_get_folder_properties (conn, &obj_folder, NULL, NULL, e_mapi_utils_get_folder_basic_properties_cb, &fbp, cancellable, mapi_error);
 		if (status) {
@@ -722,6 +787,7 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 	gco.summary = folder->summary;
 	gco.to_update = NULL;
 	gco.removed_uids = NULL;
+	gco.is_public_folder = (mapi_folder->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_PUBLIC) != 0;
 
 	if (msi->latest_last_modify <= 0) {
 		GPtrArray *known_uids;
@@ -752,6 +818,7 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 
 		gos.folder = folder;
 		gos.changes = camel_folder_change_info_new ();
+		gos.is_public_folder = gco.is_public_folder;
 
 		if (gco.removed_uids)
 			g_hash_table_foreach (gco.removed_uids, remove_removed_uids_cb, &gos);
@@ -774,6 +841,7 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 
 		gos.folder = folder;
 		gos.changes = camel_folder_change_info_new ();
+		gos.is_public_folder = gco.is_public_folder;
 
 		g_hash_table_foreach (gco.removed_uids, remove_removed_uids_cb, &gos);
 
@@ -1114,7 +1182,7 @@ mapi_folder_append_message_sync (CamelFolder *folder,
 
 	/* Convert MIME to Item */
 	conn = camel_mapi_store_get_connection (mapi_store);
-	if (e_mapi_connection_open_personal_folder (conn, fid, &obj_folder, cancellable, &mapi_error)) {
+	if (cmf_open_folder (CAMEL_MAPI_FOLDER (folder), conn, &obj_folder, cancellable, &mapi_error)) {
 		struct CamelMapiCreateData cmc;
 
 		cmc.message = message;
@@ -1161,6 +1229,7 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 	gboolean delete = FALSE, status = FALSE;
 	GSList *deleted_items, *deleted_head;
 	GSList *deleted_items_uid, *deleted_items_uid_head;
+	EMapiConnection *conn;
 
 	deleted_items = deleted_head = NULL;
 	deleted_items_uid = deleted_items_uid_head = NULL;
@@ -1169,8 +1238,10 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 
 	mapi_folder = CAMEL_MAPI_FOLDER (folder);
 	mapi_store = CAMEL_MAPI_STORE (parent_store);
+	conn = camel_mapi_store_get_connection (mapi_store);
 
 	if ((mapi_folder->camel_folder_flags & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_TRASH) {
+		mapi_object_t obj_folder;
 		GError *mapi_error = NULL;
 		GPtrArray *folders;
 		gint ii;
@@ -1195,7 +1266,11 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 		g_ptr_array_free (folders, TRUE);
 
 		camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-		status = e_mapi_connection_empty_folder (camel_mapi_store_get_connection (mapi_store), mapi_folder->folder_id, 0, cancellable, &mapi_error);
+		status = cmf_open_folder (mapi_folder, conn, &obj_folder, cancellable, &mapi_error);
+		if (status) {
+			status = e_mapi_connection_empty_folder (conn, &obj_folder, cancellable, &mapi_error);
+			e_mapi_connection_close_folder (conn, &obj_folder, cancellable, &mapi_error);
+		}
 		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 		if (status) {
@@ -1248,9 +1323,15 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 	deleted_items_uid_head = deleted_items_uid;
 
 	if (deleted_items) {
+		mapi_object_t obj_folder;
+
 		camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
-		status = e_mapi_connection_remove_items (camel_mapi_store_get_connection (mapi_store), 0, mapi_folder->folder_id, 0, deleted_items, cancellable, NULL);
+		status = cmf_open_folder (mapi_folder, conn, &obj_folder, cancellable, NULL);
+		if (status) {
+			status = e_mapi_connection_remove_items (conn, &obj_folder, deleted_items, cancellable, NULL);
+			e_mapi_connection_close_folder (conn, &obj_folder, cancellable, NULL);
+		}
 
 		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
@@ -1406,11 +1487,7 @@ mapi_folder_get_message_sync (CamelFolder *folder,
 
 	conn = camel_mapi_store_get_connection (mapi_store);
 
-	if (mapi_folder->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_PUBLIC)
-		success = e_mapi_connection_open_public_folder (conn, mapi_folder->folder_id, &obj_folder, cancellable, &mapi_error);
-	else
-		success = e_mapi_connection_open_personal_folder (conn, mapi_folder->folder_id, &obj_folder, cancellable, &mapi_error);
-
+	success = cmf_open_folder (mapi_folder, conn, &obj_folder, cancellable, &mapi_error);
 	if (success) {
 		success = e_mapi_connection_transfer_object (conn, &obj_folder, id_message, transfer_mail_object_cb, &msg, cancellable, &mapi_error);
 
@@ -1463,15 +1540,16 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 	CamelFolderChangeInfo *changes = NULL;
 	CamelServiceConnectionStatus status;
 	CamelService *service;
+	EMapiConnection *conn;
 	GPtrArray *known_uids;
-	GSList *read_items = NULL, *unread_items = NULL, *to_free = NULL, *junk_items = NULL, *deleted_items = NULL, *l;
+	GSList *read_items = NULL, *read_with_receipt = NULL, *unread_items = NULL, *to_free = NULL, *junk_items = NULL, *deleted_items = NULL, *l;
 	flags_diff_t diff, unset_flags;
 	const gchar *folder_id;
 	const gchar *full_name;
-	mapi_id_t fid, deleted_items_fid;
+	mapi_id_t fid;
 	gint i;
-	guint32 options =0;
-	gboolean is_junk_folder;
+	gboolean is_junk_folder, has_obj_folder = FALSE;
+	mapi_object_t obj_folder;
 
 	full_name = camel_folder_get_full_name (folder);
 	parent_store = camel_folder_get_parent_store (folder);
@@ -1487,9 +1565,6 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 		return TRUE;
 	}
 
-	if (((CamelMapiFolder *)folder)->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_PUBLIC)
-		options |= MAPI_OPTIONS_USE_PFSTORE;
-
 	folder_id =  camel_mapi_store_folder_id_lookup (mapi_store, full_name);
 	e_mapi_util_mapi_id_from_string (folder_id, &fid);
 
@@ -1545,6 +1620,8 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 
 			if (diff.bits & CAMEL_MESSAGE_SEEN) {
 				read_items = g_slist_prepend (read_items, mid);
+				if (flags & CAMEL_MAPI_MESSAGE_WITH_READ_RECEIPT)
+					read_with_receipt = g_slist_prepend (read_with_receipt, mid);
 				used = TRUE;
 			} else if (unset_flags.bits & CAMEL_MESSAGE_SEEN) {
 				unread_items = g_slist_prepend (unread_items, mid);
@@ -1572,28 +1649,38 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 	   Evo doesnt not take care of it, as I find that scenario to be impractical.
 	*/
 
-	if (read_items) {
+	conn = camel_mapi_store_get_connection (mapi_store);
+	has_obj_folder = cmf_open_folder (mapi_folder, conn, &obj_folder, cancellable, NULL);
+
+	if (read_items && has_obj_folder) {
 		camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-		e_mapi_connection_set_flags (camel_mapi_store_get_connection (mapi_store), 0, fid, options, read_items, 0, cancellable, NULL);
+		if (read_with_receipt)
+			e_mapi_connection_set_flags (conn, &obj_folder, read_with_receipt, CLEAR_RN_PENDING, cancellable, NULL);
+		e_mapi_connection_set_flags (conn, &obj_folder, read_items, 0, cancellable, NULL);
 		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	}
 
-	if (unread_items) {
+	if (unread_items && has_obj_folder) {
 		camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-		e_mapi_connection_set_flags (camel_mapi_store_get_connection (mapi_store), 0, fid, options, unread_items, CLEAR_READ_FLAG, cancellable, NULL);
+		e_mapi_connection_set_flags (conn, &obj_folder, unread_items, CLEAR_READ_FLAG, cancellable, NULL);
 		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	}
 
 	/* Remove messages from server*/
-	if (deleted_items) {
+	if (deleted_items && has_obj_folder) {
 		camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		if ((mapi_folder->camel_folder_flags & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_TRASH) {
-			e_mapi_connection_remove_items (camel_mapi_store_get_connection (mapi_store), 0, fid, options, deleted_items, cancellable, NULL);
+			e_mapi_connection_remove_items (conn, &obj_folder, deleted_items, cancellable, NULL);
 		} else {
 			GError *err = NULL;
+			mapi_id_t deleted_items_fid;
+			mapi_object_t deleted_obj_folder;
 
 			e_mapi_util_mapi_id_from_string (camel_mapi_store_system_folder_fid (mapi_store, olFolderDeletedItems), &deleted_items_fid);
-			e_mapi_connection_move_items (camel_mapi_store_get_connection (mapi_store), fid, options, deleted_items_fid, 0, deleted_items, cancellable, &err);
+			if (e_mapi_connection_open_personal_folder (conn, deleted_items_fid, &deleted_obj_folder, cancellable, &err)) {
+				e_mapi_connection_copymove_items (conn, &obj_folder, &deleted_obj_folder, FALSE, deleted_items, cancellable, &err);
+				e_mapi_connection_close_folder (conn, &deleted_obj_folder, cancellable, &err);
+			}
 
 			if (err) {
 				g_warning ("%s: Failed to move deleted items: %s", G_STRFUNC, err->message);
@@ -1604,14 +1691,20 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	}
 
-	if (junk_items) {
+	if (junk_items && has_obj_folder) {
 		mapi_id_t junk_fid = 0;
+		mapi_object_t junk_obj_folder;
 		GError *err = NULL;
 
-		camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-		e_mapi_util_mapi_id_from_string (camel_mapi_store_system_folder_fid (mapi_store, olFolderJunk), &junk_fid);
-		e_mapi_connection_move_items (camel_mapi_store_get_connection (mapi_store), fid, options, junk_fid, 0, junk_items, cancellable, &err);
-		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+		if (has_obj_folder) {
+			camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+			e_mapi_util_mapi_id_from_string (camel_mapi_store_system_folder_fid (mapi_store, olFolderJunk), &junk_fid);
+			if (e_mapi_connection_open_personal_folder (conn, junk_fid, &junk_obj_folder, cancellable, &err)) {
+				e_mapi_connection_copymove_items (conn, &obj_folder, &junk_obj_folder, FALSE, junk_items, cancellable, &err);
+				e_mapi_connection_close_folder (conn, &junk_obj_folder, cancellable, &err);
+			}
+			camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+		}
 
 		/* in junk_items are only emails which are not deleted */
 		deleted_items = g_slist_concat (deleted_items, g_slist_copy (junk_items));
@@ -1622,6 +1715,9 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 		}
 	}
 
+	if (has_obj_folder)
+		e_mapi_connection_close_folder (conn, &obj_folder, cancellable, NULL);
+
 	/*Remove messages from local cache*/
 	for (l = deleted_items; l; l = l->next) {
 		gchar *deleted_msg_uid = e_mapi_util_mapi_id_to_string (*((mapi_id_t *) l->data));
@@ -1667,8 +1763,6 @@ mapi_folder_transfer_messages_to_sync (CamelFolder *source,
                                        GCancellable *cancellable,
                                        GError **error)
 {
-	guint32 src_fid_options, dest_fid_options;
-
 	CamelOfflineStore *offline;
 	CamelMapiStore *mapi_store;
 	CamelFolderChangeInfo *changes = NULL;
@@ -1678,6 +1772,10 @@ mapi_folder_transfer_messages_to_sync (CamelFolder *source,
 	gint i = 0;
 	GSList *src_msg_ids = NULL;
 	gboolean success = TRUE;
+	GError *err = NULL;
+	mapi_object_t src_obj_folder, des_obj_folder;
+	gboolean copymoved = FALSE;
+	EMapiConnection *conn;
 
 	if (CAMEL_IS_MAPI_FOLDER (source)) {
 		/* make sure changed flags are written into the server */
@@ -1709,10 +1807,7 @@ mapi_folder_transfer_messages_to_sync (CamelFolder *source,
 		return FALSE;
 
 	src_mapi_folder = CAMEL_MAPI_FOLDER (source);
-	src_fid_options = (src_mapi_folder->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_PUBLIC) != 0 ? MAPI_OPTIONS_USE_PFSTORE : 0;
-
 	des_mapi_folder = CAMEL_MAPI_FOLDER (destination);
-	dest_fid_options = (des_mapi_folder->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_PUBLIC) != 0 ? MAPI_OPTIONS_USE_PFSTORE : 0;
 
 	for (i=0; i < uids->len; i++) {
 		mapi_id_t *mid = g_new0 (mapi_id_t, 1); /* FIXME : */
@@ -1722,40 +1817,38 @@ mapi_folder_transfer_messages_to_sync (CamelFolder *source,
 		src_msg_ids = g_slist_prepend (src_msg_ids, mid);
 	}
 
-	if (delete_originals) {
-		GError *err = NULL;
-
-		if (!e_mapi_connection_move_items (camel_mapi_store_get_connection (mapi_store), src_mapi_folder->folder_id, src_fid_options, des_mapi_folder->folder_id, dest_fid_options, src_msg_ids, cancellable, &err)) {
-			if (!e_mapi_utils_propagate_cancelled_error (err, error))
-				g_set_error (
-					error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
-					"%s", err ? err->message : _("Unknown error"));
-			g_clear_error (&err);
-			success = FALSE;
-		} else {
-			changes = camel_folder_change_info_new ();
-
-			for (i=0; i < uids->len; i++) {
-				camel_folder_summary_remove_uid (source->summary, uids->pdata[i]);
-				camel_folder_change_info_remove_uid (changes, uids->pdata[i]);
-			}
-			camel_folder_changed (source, changes);
-			camel_folder_change_info_free (changes);
+	conn = camel_mapi_store_get_connection (mapi_store);
 
+	if (cmf_open_folder (src_mapi_folder, conn, &src_obj_folder, cancellable, &err)) {
+		if (cmf_open_folder (des_mapi_folder, conn, &des_obj_folder, cancellable, &err)) {
+			copymoved = e_mapi_connection_copymove_items (conn, &src_obj_folder, &des_obj_folder, !delete_originals, src_msg_ids, cancellable, &err);
+			e_mapi_connection_close_folder (conn, &des_obj_folder, cancellable, &err);
 		}
-	} else {
-		GError *err = NULL;
 
-		if (!e_mapi_connection_copy_items (camel_mapi_store_get_connection (mapi_store), src_mapi_folder->folder_id, src_fid_options, des_mapi_folder->folder_id, dest_fid_options, src_msg_ids, cancellable, &err)) {
-			if (!e_mapi_utils_propagate_cancelled_error (err, error))
-				g_set_error (
-					error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
-					"%s", err ? err->message : _("Unknown error"));
-			g_clear_error (&err);
-			success = FALSE;
-		}
+		e_mapi_connection_close_folder (conn, &src_obj_folder, cancellable, &err);
 	}
 
+	if (!copymoved) {
+		if (!e_mapi_utils_propagate_cancelled_error (err, error))
+			g_set_error (
+				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				"%s", err ? err->message : _("Unknown error"));
+		g_clear_error (&err);
+		success = FALSE;
+	} else {
+		changes = camel_folder_change_info_new ();
+
+		for (i=0; i < uids->len; i++) {
+			camel_folder_summary_remove_uid (source->summary, uids->pdata[i]);
+			camel_folder_change_info_remove_uid (changes, uids->pdata[i]);
+		}
+		camel_folder_changed (source, changes);
+		camel_folder_change_info_free (changes);
+
+	}
+
+	g_clear_error (&err);
+
 	g_slist_foreach (src_msg_ids, (GFunc) g_free, NULL);
 	g_slist_free (src_msg_ids);
 
@@ -1766,6 +1859,55 @@ mapi_folder_transfer_messages_to_sync (CamelFolder *source,
 	return success;
 }
 
+static CamelFolderQuotaInfo *
+mapi_folder_get_quota_info_sync (CamelFolder *folder,
+				 GCancellable *cancellable,
+				 GError **error)
+{
+	CamelMapiStore *mapi_store;
+	CamelFolderQuotaInfo *quota_info = NULL;
+	EMapiConnection *conn;
+	uint64_t current_size = -1, receive_quota = -1, send_quota = -1;
+
+	g_return_val_if_fail (folder != NULL, NULL);
+	g_return_val_if_fail (CAMEL_IS_MAPI_FOLDER (folder), NULL);
+
+	mapi_store = CAMEL_MAPI_STORE (camel_folder_get_parent_store (folder));
+	g_return_val_if_fail (mapi_store != NULL, NULL);
+	
+	/* check for offline operation */
+	if (!camel_offline_store_get_online (CAMEL_OFFLINE_STORE (mapi_store)))
+		return NULL;
+
+	conn = camel_mapi_store_get_connection (mapi_store);
+	if (conn && e_mapi_connection_get_store_quotas (
+		conn, NULL,
+		&current_size, &receive_quota, &send_quota,
+		cancellable, error)) {
+		if (current_size != -1) {
+			if (receive_quota != -1) {
+				quota_info = camel_folder_quota_info_new (_("Receive quota"), current_size, receive_quota);
+			}
+
+			if (send_quota != -1) {
+				CamelFolderQuotaInfo *qi;
+
+				qi = camel_folder_quota_info_new (_("Send quota"), current_size, send_quota);
+				if (quota_info)
+					quota_info->next = qi;
+				else
+					quota_info = qi;
+			}
+		}
+	}
+
+	if (!quota_info)
+		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+			_("No quota information available"));
+
+	return quota_info;
+}
+
 static void
 camel_mapi_folder_class_init (CamelMapiFolderClass *class)
 {
@@ -1792,6 +1934,7 @@ camel_mapi_folder_class_init (CamelMapiFolderClass *class)
 	folder_class->refresh_info_sync = mapi_folder_refresh_info_sync;
 	folder_class->synchronize_sync = mapi_folder_synchronize_sync;
 	folder_class->transfer_messages_to_sync = mapi_folder_transfer_messages_to_sync;
+	folder_class->get_quota_info_sync = mapi_folder_get_quota_info_sync;
 }
 
 static void
@@ -1906,6 +2049,11 @@ camel_mapi_folder_new (CamelStore *store, const gchar *folder_name, const gchar
 		mapi_folder->mapi_folder_flags = msi->mapi_folder_flags;
 		mapi_folder->camel_folder_flags = msi->camel_folder_flags;
 		mapi_folder->folder_id = msi->folder_id;
+		if ((mapi_folder->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_FOREIGN) != 0) {
+			mapi_folder->priv->foreign_username = g_strdup (msi->foreign_username);
+		} else {
+			mapi_folder->priv->foreign_username = NULL;
+		}
 
 		if ((si->flags & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_TRASH)
 			folder->folder_flags |= CAMEL_FOLDER_IS_TRASH;
@@ -1920,5 +2068,11 @@ camel_mapi_folder_new (CamelStore *store, const gchar *folder_name, const gchar
 		((CamelMapiStore *) store)->summary,
 		folder_name, folder->summary);
 
+	/* sanity checking */
+	if ((mapi_folder->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_FOREIGN) != 0)
+		g_return_val_if_fail (mapi_folder->priv->foreign_username != NULL, folder);
+	if ((mapi_folder->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_PUBLIC) != 0)
+		g_return_val_if_fail (mapi_folder->priv->foreign_username == NULL, folder);
+
 	return folder;
 }
